@@ -30,7 +30,7 @@ static const char copyright[] _U_ =
     "@(#) Copyright (c) 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 2000\n\
 The Regents of the University of California.  All rights reserved.\n";
 static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/tcpdump.c,v 1.271.2.11 2008-09-25 21:50:04 guy Exp $ (LBL)";
+    "@(#) $Header: /tcpdump/master/tcpdump/tcpdump.c,v 1.283 2008-09-25 21:45:50 guy Exp $ (LBL)";
 #endif
 
 /*
@@ -275,6 +275,12 @@ static struct printer printers[] = {
 #endif
 #if defined(DLT_BLUETOOTH_HCI_H4_WITH_PHDR) && defined(HAVE_PCAP_BLUETOOTH_H)
 	{ bt_if_print, DLT_BLUETOOTH_HCI_H4_WITH_PHDR},
+#endif
+#if defined(HAVE_PCAP_USB_H) && defined(DLT_USB_LINUX)
+	{ usb_linux_print, DLT_USB_LINUX},
+#endif
+#if defined(HAVE_PCAP_USB_H) && defined(DLT_USB_LINUX_MMAPPED)
+	{ usb_linux_print, DLT_USB_LINUX_MMAPPED},
 #endif
 	{ NULL,			0 },
 };
@@ -545,6 +551,9 @@ main(int argc, char **argv)
 		case 'A':
 			++Aflag;
 			break;
+		case 'b':
+			++bflag;
+			break;
 
 #if defined(HAVE_PCAP_CREATE) || defined(WIN32)
 		case 'B':
@@ -646,11 +655,17 @@ main(int argc, char **argv)
 				if (pcap_findalldevs(&devpointer, ebuf) < 0)
 					error("%s", ebuf);
 				else {
-					for (i = 0; i < devnum-1; i++){
-						devpointer = devpointer->next;
-						if (devpointer == NULL)
-							error("Invalid adapter index");
-					}
+					/*
+					 * Look for the devnum-th entry
+					 * in the list of devices
+					 * (1-based).
+					 */
+					for (i = 0;
+					    i < devnum-1 && devpointer != NULL;
+					    i++, devpointer = devpointer->next)
+						;
+					if (devpointer == NULL)
+						error("Invalid adapter index");
 				}
 				device = devpointer->name;
 				break;
@@ -708,7 +723,7 @@ main(int argc, char **argv)
 #ifndef HAVE_LIBCRYPTO
 			warning("crypto code not compiled in");
 #endif
-			tcpmd5secret = optarg;
+			sigsecret = optarg;
 			break;
 
 		case 'n':
@@ -745,10 +760,10 @@ main(int argc, char **argv)
 
 			snaplen = strtol(optarg, &end, 0);
 			if (optarg == end || *end != '\0'
-			    || snaplen < 0 || snaplen > 65535)
+			    || snaplen < 0 || snaplen > MAXIMUM_SNAPLEN)
 				error("invalid snaplen %s", optarg);
 			else if (snaplen == 0)
-				snaplen = 65535;
+				snaplen = MAXIMUM_SNAPLEN;
 			break;
 		}
 
@@ -1073,6 +1088,7 @@ main(int argc, char **argv)
 
 	if (pcap_compile(pd, &fcode, cmdbuf, Oflag, netmask) < 0)
 		error("%s", pcap_geterr(pd));
+	free(cmdbuf);
 	if (dflag) {
 		bpf_dump(&fcode, dflag);
 		pcap_close(pd);
@@ -1128,10 +1144,10 @@ main(int argc, char **argv)
 		if (printinfo.printer == NULL) {
 			gndo->ndo_dltname = pcap_datalink_val_to_name(type);
 			if (gndo->ndo_dltname != NULL)
-				error("unsupported data link type %s",
+				error("packet printing is not supported for link type %s: use -w",
 				      gndo->ndo_dltname);
 			else
-				error("unsupported data link type %d", type);
+				error("packet printing is not supported for link type %d: use -w", type);
 		}
 		callback = print_packet;
 		pcap_userdata = (u_char *)&printinfo;
@@ -1324,7 +1340,7 @@ compress_savefile(const char *filename)
 #else
 	setpriority(PRIO_PROCESS, 0, 19);
 #endif
-	if (execlp(zflag, zflag, filename, NULL) == -1)
+	if (execlp(zflag, zflag, filename, (char *)NULL) == -1)
 		fprintf(stderr,
 			"compress_savefile:execlp(%s, %s): %s\n",
 			zflag,
